﻿namespace MesETL.App;

public class DataRecord : ICloneable
{
    /// <summary>
    /// 尝试获取一条记录的某个字段值
    /// </summary>
    /// <param name="record"></param>
    /// <param name="columnName"></param>
    /// <param name="value"></param>
    /// <returns></returns>
    /// <exception cref="InvalidOperationException"></exception>
    public static bool TryGetField(DataRecord record, string columnName, out string value)
    {
        value = string.Empty;
        if (record.Headers is null)
            throw new InvalidOperationException("Cannot get field when headers of a record have not been set.");
        var idx = IndexOfIgnoreCase(record.Headers, columnName);
        if (idx == -1)
            return false;
        value = record.Fields[idx];
        return true;
    }

    /// <summary>
    /// 获取一条记录的某个字段值
    /// TODO: 最好能优化至O(1)
    /// </summary>
    /// <param name="record"></param>
    /// <param name="columnName"></param>
    /// <returns></returns>
    /// <exception cref="InvalidOperationException"></exception>
    /// <exception cref="IndexOutOfRangeException"></exception>
    public static string GetField(DataRecord record, string columnName)
    {
        if (record.Headers is null)
            throw new InvalidOperationException("Headers have not been set.");
        var idx = IndexOfIgnoreCase(record.Headers, columnName);
        if (idx is -1)
            throw new IndexOutOfRangeException(
                $"Column name '{columnName}' not found in this record, table name '{record.TableName}'.");
        return record.Fields[idx];
    }
    
    private static int IndexOfIgnoreCase(IList<string> list, string value)
    {
        var idx = -1;
        for (var i = 0; i < list.Count; i++)
        {
            if (list[i].Equals(value, StringComparison.OrdinalIgnoreCase)) 
            {
                idx = i;
                break;
            }
        }

        return idx;
    }

    /// <summary>
    /// 字段列表
    /// </summary>
    public IList<string> Fields { get; }
    /// <summary>
    /// 表头列表
    /// </summary>
    public IList<string> Headers { get; }
    /// <summary>
    /// 来源表名
    /// </summary>
    public string TableName { get; }
    /// <summary>
    /// 需要输出的数据库
    /// </summary>
    public string? Database { get; set; }
    /// <summary>
    /// 所有字段的总字符数量
    /// </summary>
    public long FieldCharCount { get; }
    /// <summary>
    /// 忽略这个记录，不会被输出
    /// </summary>
    public bool Ignore { get; set; }


    public DataRecord(IEnumerable<string> fields, string tableName, IEnumerable<string> headers, string? database = null)
    {
        Fields = fields.ToList();
        TableName = tableName;
        Headers = headers.ToList();
        Database = database;
        
        if (Fields.Count != Headers.Count)
            throw new ArgumentException(
                $"The number of fields does not match the number of headers. Expected: {Headers.Count} Got: {Fields.Count} Fields: {string.Join(',', Fields)}",
                nameof(fields));
        
        FieldCharCount = Fields.Sum(x => (long)x.Length);
    }

    /// <summary>
    /// 使用索引访问字段
    /// </summary>
    /// <param name="index"></param>
    public string this[int index]
    {
        get => Fields[index];
        set => Fields[index] = value;
    }

    /// <summary>
    /// 使用列名访问字段，不区分大小写
    /// </summary>
    /// <param name="columnName"></param>
    public string this[string columnName]
    {
        get => GetField(this, columnName);
        set => SetField(columnName, value);
    }

    /// <summary>
    /// 尝试获取字段值
    /// </summary>
    /// <param name="columnName"></param>
    /// <param name="value"></param>
    /// <returns></returns>
    public bool TryGetField(string columnName, out string value) => TryGetField(this, columnName, out value);

    /// <summary>
    /// 为一个字段赋值
    /// </summary>
    /// <param name="columnName"></param>
    /// <param name="value"></param>
    /// <returns></returns>
    public bool SetField(string columnName, string value) => SetField(this, columnName, value);

    /// <summary>
    /// 
    /// </summary>
    /// <param name="record"></param>
    /// <param name="columnName"></param>
    /// <param name="value"></param>
    /// <returns></returns>
    /// <exception cref="InvalidOperationException"></exception>
    /// <exception cref="IndexOutOfRangeException"></exception>
    public static bool SetField(DataRecord record, string columnName, string value)
    {
        // 表头检查
        if (record.Headers is null)
            throw new InvalidOperationException("记录的表头尚未设置，无法赋值");
        var idx = IndexOfIgnoreCase(record.Headers, columnName);
        if (idx is -1)
            throw new IndexOutOfRangeException(
                $"列 '{columnName}' 不存在于该纪录中，表名 '{record.TableName}");
        record.Fields[idx] = value;
        return true;
    }
    
    public void RemoveField(string columnName)
    {
        var idx = IndexOfIgnoreCase(Headers, columnName);
        if (idx == -1)
            throw new InvalidOperationException($"{TableName}: 列名 '{columnName}' 不存在");
        
        Fields.RemoveAt(idx);
        Headers.Remove(columnName);
    }

    public bool HeaderExists(string columnName) => IndexOfIgnoreCase(Headers, columnName) != -1;
    public object Clone()
    {
        return new DataRecord(new List<string>(Fields), TableName, new List<string>(Headers), Database);
    }
}